#ifndef _XRDDPMOFS__H_
#define _XRDDPMOFS__H_
/******************************************************************************/
/*                                                                            */
/*                          X r d D P M O f s . h h                           */
/*                                                                            */
/* DPM handling and various by David Smith, CERN                              */            
/* Other parts:                                                               */
/*                                                                            */
/* (c) 2006 by the Board of Trustees of the Leland Stanford, Jr., University  */
/*                            All Rights Reserved                             */
/*   Produced by Andrew Hanushevsky for Stanford University under contract    */
/*              DE-AC02-76-SFO0515 with the Department of Energy              */
/******************************************************************************/

#include "XrdSys/XrdSysError.hh"
#include "XrdSys/XrdSysLogger.hh"
#include "XrdSys/XrdSysPthread.hh"
#include "XrdSec/XrdSecInterface.hh"
#include "XrdSfs/XrdSfsAio.hh"
#include "XrdSfs/XrdSfsNative.hh"
#include "XrdOuc/XrdOucEnv.hh"
#include "XrdOuc/XrdOucName2Name.hh"

class XrdOucStream;

#include "dpns_api.h"

/******************************************************************************/
/*                   X r d D P M O f s D i r e c t o r y                      */
/******************************************************************************/
  
class XrdDPMOfsDirectory : public XrdSfsDirectory
{
public:

        int         open(const char              *dirName,
			 const XrdSecEntity      *client,
                         const char              *opaque = 0);

	const char *nextEntry();

        int         close();

const   char       *FName();

                    XrdDPMOfsDirectory(XrdSfsFileSystem *xf, char* user=0);

                   ~XrdDPMOfsDirectory();

private:
        XrdSfsDirectory *m_nextXrdOfsDirectory;
const   char            *tident;
};

/******************************************************************************/
/*                        X r d D P M O f s F i l e                           */
/******************************************************************************/
  

class XrdDPMOfsFile : public XrdSfsFile
{
public:

        int            open(const char                *fileName,
                                  XrdSfsFileOpenMode   openMode,
                                  mode_t               createMode,
                            const XrdSecEntity        *client,
	                    const char                *info = 0);

        int            close();

        const char    *FName();

        int            getMmap(void **Addr, off_t &Size);

        int            read(XrdSfsFileOffset   fileOffset,
 	                    XrdSfsXferSize     preread_sz);

        XrdSfsXferSize read(XrdSfsFileOffset   fileOffset,
                            char              *buffer,
			    XrdSfsXferSize     buffer_size);

        int            read(XrdSfsAio *aioparm);

        XrdSfsXferSize write(XrdSfsFileOffset   fileOffset,
                             const char        *buffer,
			     XrdSfsXferSize     buffer_size);

        int            write(XrdSfsAio *aioparm);

        int            sync();

        int            sync(XrdSfsAio *aiop);

        int            stat(struct stat *buf);

        int            truncate(XrdSfsFileOffset   fileOffset);

        int            getCXinfo(char cxtype[4], int &cxrsz);

                       XrdDPMOfsFile(XrdSfsFileSystem *xf, char* user=0);

                      ~XrdDPMOfsFile();

private:
const   char           *tident;
        void           setInUse();
        void           clearInUse();

        void           doDPMDone(const bool &isError) const;
        void           doDPMDone(const bool &usingCgi, const bool &isPut,
                                 const char *path, const char *token, const bool &isError) const;

        XrdSysMutex    openMutex;

        // these are associated to the OfsFile instance after an open until a close
        //
        bool           m_inUse;
        char           m_fullpath[CA_MAXPATHLEN+1];
        char           m_tidypath[CA_MAXPATHLEN+1];
        char           m_sfn[CA_MAXSFNLEN+1];
        char          *m_info;
const   XrdSecEntity  *m_client;
        bool           m_usingCgi, m_isPut;
        char           m_rToken[CA_MAXDPMTOKENLEN+1];

        XrdSfsFile    *m_nextXrdOfsFile;
};

/******************************************************************************/
/*                          X r d D P M N a t i v e                           */
/******************************************************************************/
  
class XrdDPMOfs : public XrdSfsFileSystem
{
friend class XrdDPMOfsDirectory;
friend class XrdDPMOfsFile;
public:

// Object Allocation Functions
//
        XrdSfsDirectory *newDir(char *user=0)
                        {return (XrdSfsDirectory *)new XrdDPMOfsDirectory(m_nextXrdOfsFS,user);}

        XrdSfsFile      *newFile(char *user=0)
                        {return      (XrdSfsFile *)new XrdDPMOfsFile(m_nextXrdOfsFS,user);}

// Other Functions
//
        int            chmod(const char             *Name,
                                   XrdSfsMode        Mode,
                                   XrdOucErrInfo    &out_error,
                       	     const XrdSecEntity     *client,
                             const char             *opaque = 0);

        int            exists(const char                *fileName,
                                    XrdSfsFileExistence &exists_flag,
                                    XrdOucErrInfo       &out_error, 
	                      const XrdSecEntity        *client,
                              const char                *opaque = 0);

        int            fsctl(const int               cmd,
                             const char             *args,
                                   XrdOucErrInfo    &out_error,
			     const XrdSecEntity     *client);

        int            getStats(char *buff, int blen);

const   char          *getVersion();

        int            mkdir(const char             *dirName,
                                   XrdSfsMode        Mode,
                                   XrdOucErrInfo    &out_error,
                             const XrdSecEntity     *client,
                             const char             *opaque = 0);

        int            prepare(      XrdSfsPrep       &pargs,
                                     XrdOucErrInfo    &out_error,
                               const XrdSecEntity     *client = 0);

        int            rem(const char             *path,
		                 XrdOucErrInfo    &out_error,
		           const XrdSecEntity     *client,
		           const char             *info = 0);

        int            remdir(const char             *dirName,
                                    XrdOucErrInfo    &out_error,
                              const XrdSecEntity     *client,
                              const char             *info = 0);

        int            rename(const char             *oldFileName,
                              const char             *newFileName,
                                    XrdOucErrInfo    &out_error,
                              const XrdSecEntity     *client,
                              const char             *infoO = 0,
                              const char             *infoN = 0);

        int            stat(const char             *path,
                                  struct stat      *buf,
                                  XrdOucErrInfo    &einfo,
			    const XrdSecEntity     *client,
                            const char             *info = 0);

        int            stat(const char             *path,
                                  mode_t           &mode,
                                  XrdOucErrInfo    &einfo,
			    const XrdSecEntity     *client,
                            const char             *info = 0);

        char           m_FSLib[1024];
        char           m_principal[CA_MAXCSECNAMELEN+1];
        char           m_stagersuperuser[128];

        XrdSfsFileSystem *m_nextXrdOfsFS;

        // Management functions
        //
        int            Configure(XrdSysError &);

        // Error buffer for dpns/dpm API
        //
        char           m_apibuf[1024];

// configuration
        char           *ConfigFN;       //    ->Configuration filename

protected:

static  int            Emsg(const char *, XrdOucErrInfo  &, int, const char *x,
                            const char *y="");

private:
        int            xtrace(XrdOucStream &, XrdSysError &);
};

/******************************************************************************/
/*                            N a m e M a p p e r                             */
/******************************************************************************/
class XrdDPMName2Name : public XrdOucName2Name
{
public:
        int lfn2pfn(const char *lfn, char *buff, int blen);
        int lfn2rfn(const char *lfn, char *buff, int blen);
        int pfn2lfn(const char *pfn, char *buff, int blen);

             XrdDPMName2Name() {}
            ~XrdDPMName2Name() {}
};
#endif
